--*********************************************
-- Desc: 游戏数据
-- File: GameData.lua
-- Author: zhouhaifeng
-- Date: 2014-03-31 09:53:18
--*********************************************

ERROR_INVALID_FILE_CONTENTS = -1
ERROR_HASH_MISS_MATCH       = -2
ERROR_STATE_FILE_NOT_FOUND  = -3

local GameData = {}

local UITLS_DEBUG = false

local crypto = require(cc.PACKAGE_NAME .. ".crypto")
local json   = require(cc.PACKAGE_NAME .. ".json")

local encodeSign    = "=QP="
local configFiles   = {}

local function isEncodedContents_(contents)
    return string.sub(contents, 1, string.len(encodeSign)) == encodeSign
end

local function encode_(values,secretKey)
    local s = json.encode(values)
    local hash = crypto.md5(s..secretKey)
    local contents = json.encode({h = hash, s = s})
    return encodeSign..contents
end

local function decode_(fileContents,secretKey)
    local contents = string.sub(fileContents, string.len(encodeSign) + 1)
    local j = json.decode(contents)

    if type(j) ~= "table" then
        echoError("GameData.decode_() - invalid contents")
        return {errorCode = ERROR_INVALID_FILE_CONTENTS}
    end

    local hash,s = j.h, j.s
    local testHash = crypto.md5(s..secretKey)
    if testHash ~= hash then
        echoError("GameData.decode_() - hash miss match")
        return {errorCode = ERROR_HASH_MISS_MATCH}
    end

    local values = json.decode(s)
    if type(values) ~= "table" then
        echoError("GameData.decode_() - invalid state data")
        return {errorCode = ERROR_INVALID_FILE_CONTENTS}
    end

    return {values = values}
end

----------------------------------------

function GameData.init(key , eventListener_, stateFilename_, secretKey_)
   local config = {
        stateFilename = nil,
        eventListener = nil,
        secretKey = nil,
    }

    if type(eventListener_) ~= "function" then
        echoError("GameData.init() - invalid eventListener")
        return false
    end

    config.eventListener = eventListener_

    if type(stateFilename_) == "string" then
        config.stateFilename = stateFilename_
    end

    if type(secretKey_) == "string" then
        config.secretKey = secretKey_
    end

    configFiles[key] = config

    config.eventListener({
        name     = "init",
        filename = GameData.getGameDataPath(key),
        encode   = type(config.secretKey) == "string"
    })

    return true
end

function GameData.load(key)
    local config = configFiles[key]
    local filename = GameData.getGameDataPath(key)

    if not io.exists(filename) then
        -- echoInfo("GameData.load() - file \"%s\" not found", filename)
        return config.eventListener({name = "load", errorCode = ERROR_STATE_FILE_NOT_FOUND})
    end

    local contents = io.readfile(filename)

    if not UITLS_DEBUG then
        contents = crypto.decodeBase64(contents)
    end
   
    -- echoInfo("GameData.load() - get values from \"%s\"", filename)


    local values
    local encode = false

    if config.secretKey and isEncodedContents_(contents) then
        local d = decode_(contents,config.secretKey)
        if d.errorCode then
            return config.eventListener({name = "load", errorCode = d.errorCode})
        end

        values = d.values
        encode = true
    else
        values = json.decode(contents)
        if type(values) ~= "table" then
            echoError("GameData.load() - invalid data")
            return config.eventListener({name = "load", errorCode = ERROR_INVALID_FILE_CONTENTS})
        end
    end

    return config.eventListener({
        name   = "load",
        values = values,
        encode = encode,
        time   = os.time()
    })
end

function GameData.save(key,newValues)
    local config = configFiles[key]
    local values = config.eventListener({
        name   = "save",
        values = newValues,
        encode = type(GameData.secretKey) == "string"
    })
    if type(values) ~= "table" then
        echoError("GameData.save() - listener return invalid data")
        return false
    end

    local filename = GameData.getGameDataPath(key)
    local ret = false
    local contents = ""
    if config.secretKey then
        contents = encode_(values,config.secretKey) 
        if not UITLS_DEBUG then
             contents = crypto.encodeBase64(contents)
        end
        ret = io.writefile(filename, contents)
    else
        contents = json.encode(values)

        if type(contents) == "string" then
            if not UITLS_DEBUG then
                contents = crypto.decodeBase64(contents)
            end
           ret = io.writefile(filename, contents)
        end
    end
    -- echoInfo("GameData.save() - update file \"%s\" %s", filename , contents)
    return ret
end

function GameData.getGameDataPath(key)
    local config = configFiles[key]
    return string.gsub(device.writablePath, "[\\\\/]+$", "") .. "/" .. config.stateFilename
end

return GameData
